#!/bin/bash

# Version 23/12/2011
# $Revision: 1.13 $
# UDA'Software 2010

# Questo script serve per aggiungere una sentinella dell'Events
# Manager, aiutando l'utente nell'operazione.

# Viene generata una callback temporizzata che invochera' questo
# script periodicamente alla ricerca del file nel quale sara' stata
# salvata la sentinella da caricare nello script.

read_ripetizioni() {
    clear
    cat<<EOF
********************************************************************************
RIPETIZIONI
DOMANDA : Quante volte vuoi che la Sentinella venga presa in considerazione dal
          sistema ?
RISPOSTA | SIGNIFICATO
 s       | SEMPRE:  La Sentinella verra' valutata sempre, verificando se
                    l'EVENTO e la CONDIZIONE soddisfano il contesto.
 e       | ELIMINA: La Sentinella verra' valutata una sola volta, quindi,
                    automaticamente eliminata se soddisfatta.
 #       | Val.Num: Un valore numerico maggiore o uguale a 1 che determinera'
                    il numero valutazioni massime (ex: 3)

Inserire il numero di RIPETIZIONI ( s /#/e):
EOF
    read ripetizioni
    if [ "${ripetizioni}" == "s" ] || [ "${ripetizioni}" == "S" ] || [ "${ripetizioni}" == "" ]; then
	ripetizioni=0
    elif [ "${ripetizioni}" == "e" ] || [ "${ripetizioni}" == "E" ]; then
	ripetizioni="?"
    elif [ "${ripetizioni/[0-9]*/}" != "" ]; then
	echo "/ECHO %C4,1Numero di ripetizioni ${ripetizioni} non valido, impostato in automatico il valore S." >> /tmp/buduscript_sentinel.dat
	ripetizioni=0
    fi
}

read_evento() {
    clear
    cat<<EOF | more
********************************************************************************
EVENTO
DOMANDA : A quale evento deve reagire la Sentinella ?
RISPOSTA  | SIGNIFICATO
 jserver  | Quando ci si connette ad un Server IRC
 jchannel | Quando si entra in un canale (o quando lo fanno altri)
 pchannel | Quando si esce da un canale (o quando lo fanno altri)
 pserver  | Quando ci si disconnette da un Server IRC (o lo fanno altri)
 kick     | Quando viene effettuato un kick (da e per chiunque)
 who      | Quando si effettua un WHO su un utente
 chgnick  | Quando si cambia nickname (o lo fanno altri)
 text     | Quando altri scrivono in un canale
 badword  | Quando altri scrivono una parolaccia (vedi: /budus help badwordfile)
 badnick  | Quando altri usano nick proibiti (vedi: /budus help badnickfile)
 query    | Quando altri scrivono nella propria QUERY privata
 ctcp     | Quando si ricevono messaggi CTCP
 mode     | Quando qualcuno modifica la modalita' propria o altrui
 flood    | Quando altri superano i parametri anti-flood
 notify   | Quando si riceve una notifica (vedi /HELP NOTIFY)
 invite   | Quando si riceve un invito
 other    | Qualsiasi Evento non gestito dal plugin (diverso da quelli sopra)
 all      | Qualsiasi Evento (TUTTI, compresi quelli speciali)
 spcl     | Evento creato dall'Utente

Evento (jserver/jchannel/pchannel/pserver/kick/who/chgnick/text/badword/badnick/query/ctcp/mode/flood/notify/invite/other/ all /spcl):
EOF
    read evento
    if [ "${evento}" == "spcl" ]; then
	clear
	cat<<EOF
********************************************************************************
EVENTO SPECIALE
DOMANDA : Come si deve chiamare l'evento speciale ?
SUGGER. : Gli eventi speciali, cioe' quelli creati dall'utente, possono avere
          qualsiasi nome, purche' non contenga spazi.
          Possono anche essere utilizzate le variabili disponibili
          nell'Events Manager, come: \$lu, \$ru, \$rhost, \$channel, \$server, \$data,
          \$all, \$event, \$nick, \$address, \$database
          Come ad esempio: mioevento_\$nick_\$server
          ("spcl_" verra' aggiunto in testa automaticamente da questa procedura)
          Lo scopo degli eventi speciali e' quello di concatenare sentinelle
          in modo che, per svolgere una determinata azione, debba essere
          soddisfatta piu' di una condizione.
          Per realizzare questo meccanismo e' necessario che esista almeno una
          sentinella che reagisce ad un evento base (praticamente NON speciale)
          la quale deve, come azione, avere proprio la generazione dell'evento
          speciale che si va a creare.
          Per generare un evento, si ricorda che l'azione della sentinella
          suddetta dovra' essere analogo (ovviamente personalizzato) a questo:
          /event event spcl_mioevento \$event_params

Inserire il nome dell'evento speciale:
EOF
	read evento	
	evento="spcl_${evento/ /}"
    elif [ "$(echo "${evento}" | grep "^\(jserver\|jchannel\|pchannel\|pserver\|kick\|who\|chgnick\|text\|badword\|badnick\|query\|ctcp\|mode\|flood\|notify\|invite\|other\|all\)$")" == "" ]; then
	echo "/ECHO %C4,1Evento ${evento} non valido, impostato in automatico al valore ALL." >> /tmp/buduscript_sentinel.dat
	evento="all"
    fi
}

read_condizione() {
    clear
    cat<<EOF | more
********************************************************************************
CONDIZIONE
DOMANDA : Quale equazione deve essere soddisfatta affinche' la Sentinella entri
          in azione ?
SUGGER. : La condizione rappresenta il metodo con il quale una Sentinella
          stabilisce se deve eseguire la propria azione o meno.
          Cio' avviene per mezzo di un'equazione nella quale vengono confrontati
          una costante e una variabile (o due variabili).
          Ogni Evento imposta opportunamente delle variabili che possono essere
          confrontate con valori fissi - costanti - decisi dall'utente.
          Ad esempio se si vuole verificare per l'evento TEXT che la sua
          variabile \$data contenga la parola 'prova' dovremo scrivere:
                                     prova='\$data'
          se si desidera che sia esclusivamente contenuta la parola 'prova' e
          nient'altro, allora bisognera' usare gli apici, in questo modo:
                                    'prova'='\$data'
          se, ancora, la parola 'prova' deve trovarsi all'inizio, avremo:
                                    'prova='\$data'
          Le variabili disponibili sono: \$lu, \$ru, \$rhost, \$channel, \$hostname
          \$server, \$data, \$all, \$nick, \$address, \$database, \$number
          e l'equazione si puo' scrivere con 2 sintassi diverse (e opposte):
                      costante=variabile  oppure  costante!variabile
(Premere la BARRA SPAZIATRICE per proseguire)
          Il sistema consente anche l'utilizzo delle Espressioni Regolari POSIX
          al posto della costante, a patto che NON inizi con l'apice ' .
          Esempi di condizioni semplici:
          'irc.mioserver.net'='\$server'
          '#miocanale'='\$channel'
          '\$nick'='\$ru'
          'BuDuS'='\$ru'
          'frase da intercettare'='\$data'

          Esempio di condizione complessa:
          'irc.mioserver.dat:#miocanale:BuDuS'='\$server:\$channel:\$ru'

          Esempi di condizioni con espressioni regolari:
          ^!miocomando(\$_|\$)='\$data'
          (nickname1|nickname2)='\$ru'
          (\$_|^)[Cc][Ii][Aa][Oo]([\$_,;]|\$)='\$data'

          I Caratteri significativi per le espressioni sono:
          ( | ) [ ] { , } ? * ^ \$

          Per identificare una condizione SEMPRE VERA, utilizzare il solo
          carattere '*' (asterisco).

(Premere la BARRA SPAZIATRICE per proseguire)
          Esempi di condizioni semplici:
          'irc.mioserver.net'='\$server'
          '#miocanale'='\$channel'
          '\$nick'='\$ru'
          'BuDuS'='\$ru'
          'frase da intercettare'='\$data'

          Il significato delle variabili:
          \$lu           -> Nickname dell'utente che esegue l'evento
          \$ru           -> Nickname dell'utente che subisce l'evento
          \$rhost        -> Hostname dell'utente che subisce l'evento
          \$channel      -> Canale nel quale viene svolto l'evento
          \$hostname     -> Network nel quale viene svolto l'evento
          \$server       -> Server nel quale viene svolto l'evento
          \$data         -> Dati dell'evento (in genere e' un testo)
          \$all          -> Tutte le variabili messe assieme
          \$event        -> Nome dell'evento
          \$nick         -> Nickname dell'utente locale
          \$address      -> Indirizzo dell'utente locale
          \$database     -> Percorso del database degli utenti
          \$dcc_database -> Percorso del database connessioni DCC
          \$number       -> Numero della Sentinella eseguita

Inserire la Condizione:
EOF
    read condizione
    condizione=$(echo "${condizione}" | sed 's/ /$_/g')
    if [ "${condizione}" == "" ]; then
	echo "/ECHO %C4,1La Condizione non puo' essere nulla, verra' impostata automaticamente." >> /tmp/buduscript_sentinel.dat
	condizione="'\$all'='\$all'"
    elif [ "$(echo "${condizione}" | grep -F -e '=' -e '!')" == "" ] && [ "_${condizione}_" != "_*_" ]; then
	echo "/ECHO %C4,1Condizione ${condizione} non valida, manca l'operatore." >> /tmp/buduscript_sentinel.dat
	echo "/ECHO %C4,1Trasformata in automatico in: ${condizione}='\$all'" >> /tmp/buduscript_sentinel.dat
	condizione="${condizione}='\$all'"
    fi
    
    if [ "$(echo "${condizione}" | grep ^\')" == "" ]; then
	if [ "$(echo "${condizione}" | grep -F "|" | sed s/\(.*\|.*\)//)" != "" ]; then
	    echo "/ECHO %C4,1Condizione ${condizione} pericolosa, l'Espressione Regolare puo' comportare soluzioni anomale." >> /tmp/buduscript_sentinel.dat
	fi
    fi
}

read_azione() {
    clear
    cat<<EOF
********************************************************************************
AZIONE
DOMANDA : Quale comando deve essere eseguito quando la Sentinella e'
          soddisfatta ?
SUGGER. : L'azione puo' essere un qualsiasi comando IRC o BuDuScRiPt,
          scritto esattamente come verrebbe inserito al prompt dei comandi di
          XChat o, addirittura, puo' essere la generazione di un
          Evento Speciale, creato quindi dall'utente.
          Possono anche essere utilizzate le variabili disponibili
          nell'Events Manager, come: \$lu, \$ru, \$rhost, \$channel, \$server, \$data,
          \$all, \$nick, \$address, \$database
          E' possibile specificare piu' comandi separandoli dalla sequenza di
          caratteri '\;', come ad esempio:
          /ECHO Prova di un'azione\;/SAY questa azione ha il valore \$data
          Per creare un evento speciale, si puo' usare un comando come questo:
          /event event spcl_nomeevento \$event_params
          dove 'nomeevento' dovra' essere il nome di un evento successivamente
          creato (cioe' si dovra' creare una sentinella che reagisca a
          spcl_nomeevento).

Inserire l'Azione:
EOF
    read azione
    if [ "${azione}" == "" ]; then
	echo "/ECHO %C4,1Non e' possibile avere un'azione nulla, verra' impostata automaticamente." >> /tmp/buduscript_sentinel.dat
	azione="/ECHO Azione non determinata !"
    fi
}

make_sentinel() {
    clear
    cat<<EOF
********************************************************************************
 Questa procedura ha il compito di aiutarti nella creazione di una sentinella
funzionante, il qui scopo, e' quello di automatizzare l'esecuzione di una o piu'
operazioni in funzione di un ben determinato contesto.

 Le Sentinelle hanno la sintassi:

                      RIPETIZIONI EVENTO CONDIZIONE AZIONE

 Le RIPETIZIONI rappresentano quante volte una Sentinella deve essere presa in
esame (valutata), l'EVENTO indica in quale contesto deve operare la Sentinella
(quando entra un utente, quando qualcuno scrive, ecc...), la CONDIZIONE e'
cio' che determina se l'AZIONE deve essere eseguita o meno dove,
quest'ultima, rappresenta il Comando IRC da eseguire.

 In ordine, andro' a chiederti queste 4 cose.

 Anche se questa procedura facilita molto la creazione di sentinelle e' comunque
consigliata la lettura della documentazione inclusa con lo script, al fine di
sfruttarne al meglio le possibilita' offerte.

Premi INVIO per continuare
********************************************************************************
EOF
    read
    
    clear
    read_ripetizioni
    read_evento
    read_condizione
    read_azione
    cat<<EOF
********************************************************************************
                            Sentinella Risultante
********************************************************************************
RIPETIZIONI: ${ripetizioni} (Il valore '0' rappresenta SEMPRE e '?' ELIMINA)

EVENTO     : ${evento}

CONDIZIONE : ${condizione}

AZIONE     : ${azione}
********************************************************************************
Comando BuDuScRiPt completo:
 /event add ${ripetizioni} ${evento} ${condizione} ${azione}
********************************************************************************
 La sentinella che si sta per inserire in BuDuScRiPt avra' i parametri elencati
sopra e, salvo non venga salvata (/event save) o eliminata, rimarra' disponibile
fino a quando non si chiude il programma.

 Si ricorda che alcuni eventi, quello TEXT in particolare, reagiscono
esclusivamente a cio' che viene prodotto dagli altri, quindi non e' possibile
fare dei test scrivendo in proprio al fine di verificarne il funzionamento.
********************************************************************************

Vuoi salvarla in BuDuScRiPt ? (S/n)
EOF
    read salvataggio
    if [ "${salvataggio}" == "s" ] || [ "${salvataggio}" == "S" ] || [ "${salvataggio}" == "" ]; then
	printf "/event chk %s %s %s %s\n" "${ripetizioni}" "${evento}" "${condizione}" "${azione}" >> /tmp/buduscript_sentinel.dat
	printf "/event add %s %s %s %s\n" "${ripetizioni}" "${evento}" "${condizione}" "${azione}" >> /tmp/buduscript_sentinel.dat
    else
	touch /tmp/buduscript_sentinel.del
    fi
}

if [ ! -d /tmp ]; then
    echo "/ECHO ERRORE: Impossibile procedere senza la cartella /tmp !"
    
    exit 1;
fi

if [ "$1" == "callback" ]; then
    if [ -f /tmp/buduscript_sentinel.dat ] && [ "$(tail -n1 /tmp/buduscript_sentinel.dat | grep -F "/event add ")" != "" ]; then
	echo "/ECHO Sentinella da caricare intercettata."
	
	cat /tmp/buduscript_sentinel.dat
	echo "/ECHO %B$(tail -n1 /tmp/buduscript_sentinel.dat)"
	echo "/event list"
	
	rm -f /tmp/buduscript_sentinel.dat
	rm -f /tmp/buduscript_sentinel.lck
	
	exit 0;
    elif [ -f /tmp/buduscript_sentinel.del ]; then
	echo "/ECHO Creazione sentinella abortita."
	
	rm -f /tmp/buduscript_sentinel.del
	rm -f /tmp/buduscript_sentinel.lck
    elif [ -f /tmp/buduscript_sentinel.lck ]; then
	echo "/TIMER -refnum 2255 3 budus pipe_script_silent $HOME/.xchat2/buduscript/buduscript_script__sentinel_add.sh callback"
    else
	echo "/ECHO Creazione Sentinella rimasta in sospeso, terminata ora !"
	
	exit 1;
    fi
elif [ "$1" == "makesentinel" ]; then
    # CREAZIONE DELLA SENTINELLA
    echo "/ECHO %C3,1Caricamento della sentinella creata:" > /tmp/buduscript_sentinel.dat
    make_sentinel
    
    if [ ! -f /tmp/buduscript_sentinel.dat ]; then
	touch /tmp/buduscript_sentinel.del
    fi
    
    exit 0;
else
    if [ ! -f /tmp/buduscript_sentinel.lck ]; then
	touch /tmp/buduscript_sentinel.lck
	if [ ! -f /tmp/buduscript_sentinel.lck ]; then
	    echo "/ECHO ERRORE: Impossibile procedere senza permesso in scrittura nella cartella /tmp !"
	    
	    exit 1;
	fi
	
	echo "/exec xterm -e \"$0 makesentinel\" &"
	echo "/TIMER -refnum 2255 3 budus pipe_script_silent $HOME/.xchat2/buduscript/buduscript_script__sentinel_add.sh callback"
    else
	echo "/ECHO AVVERTIMENTO: Non e' possibile creare piu' Sentinelle contemporaneamente, finire l'altro processo."
	echo "/ECHO               Se l'altro processo e' stato chiuso in modo anomalo, rilanciare questo comando."
	
	rm -f /tmp/buduscript_sentinel.lck
    fi    
fi
